package io.renren.modules.sys.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import io.renren.common.utils.PageUtils;
import io.renren.modules.sys.dao.algs.AlgTrainMapper;
import io.renren.modules.sys.dao.algs.AlgTrainVoMapper;
import io.renren.modules.sys.dao.algs.AlgorithmVoMapper;
import io.renren.modules.sys.entity.algs.AlgTrain;
import io.renren.modules.sys.entity.algs.AlgTrain_Vo;
import io.renren.modules.sys.entity.algs.Algorithm;
import io.renren.modules.sys.entity.algs.Algorithm_Vo;
import io.renren.modules.sys.service.AlgTrainService;
import io.renren.modules.sys.service.AlgsService;
import io.renren.modules.sys.service.CategoryService;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.Arrays;
import java.util.List;
import java.util.Map;

/**
 * @Author: Ivan Q
 * @Date: 2021/6/11 16:09
 */
@Service("algTrainService")
public class AlgTrainServiceImpl extends ServiceImpl<AlgTrainMapper, AlgTrain> implements AlgTrainService {
    @Autowired
    AlgTrainVoMapper algTrainVoMapper;
    @Autowired
    AlgTrainMapper algTrainMapper;
    @Autowired
    CategoryService categoryService;


    //获取模板列表,使用的是@select连表查询
    //但是怎么把搜索功能加进去呢？？？？
    @Override
    public PageUtils queryPageOnUse(Map<String, Object> params) {
        //获取搜索框搜索的模板名称
        String missName = (String)params.get("missName");
        String classificationtag = (String)params.get("classificationtag");
        Long categoryId = categoryService.getByName(classificationtag);
        byte missStatus = -1;
        System.out.println("@@@@@@@@@@@@@@@@@@@@@@@@@@@@@");
        System.out.println(params.get("missStatus"));
        if(params.get("missStatus") !=null){
            String t = (String) params.get("missStatus");
            if(t.equals("0"))
                missStatus = 0;
            else missStatus = 1;
        }
        //获取当前登录的用户,如果是管理员的话，uid就为null
        Long uid = (Long) params.get("currentUserId");
        List<AlgTrain_Vo> algTrainInfo;
        List<AlgTrain_Vo> algTrainAllInfo;
        int pageSize = Integer.valueOf((String) params.get("limit"));
        int pageIndex = Integer.valueOf((String) params.get("page"));

//        此方法如果修改,则也需要修改下面的获取总页数的方法
        algTrainInfo = algTrainVoMapper.getAlgTrainInfo(new QueryWrapper<Algorithm_Vo>()
                        .like(StringUtils.isNotBlank(missName), "miss_name", missName)
                        //根据登录的用户展示他自己的模板
                        .eq(uid != null, "uid", uid)
                        .eq(classificationtag != null, "category_id", categoryId)
                        .eq(params.get("missStatus") != null, "miss_status", missStatus)
                        .orderByDesc("algorithm_training_id")
//                使用limit用于分页,下面的函数没有limit进行分页
                        .last("limit " + (pageIndex - 1) * pageSize + "," + pageSize + ";")
        );

        //用于查全部，获取总页数。
        algTrainAllInfo = algTrainVoMapper.getAlgTrainInfo(new QueryWrapper<Algorithm_Vo>()
                .like(StringUtils.isNotBlank(missName), "miss_name", missName)
                //根据登录的用户展示他自己的模板
                .eq(uid != null, "uid", uid)
                .eq(classificationtag != null, "category_id", categoryId)
                .eq(params.get("missStatus") != null, "miss_status", missStatus)
        );

        for (AlgTrain_Vo vo : algTrainInfo) {
            vo.setCategoryName(categoryService.getById(vo.getCategoryId()).getCategoryName());
        }

        //弃用的方法
        //这个方法：根据当前登录的用户id返回只有当前用户的数据，但是没有结合搜索框搜索
        //templateInfo = algorithmTemplateVoMapper.getTemplateInfoById(uid);
        return new PageUtils(algTrainInfo, algTrainAllInfo.size(), pageSize, pageIndex);

    }

    @Override
    public void deleteBatch(Long[] alglIds) {
        this.removeByIds(Arrays.asList(alglIds));
    }

    @Override
    public Long querycategoryId(Long algId) {
        return null;
    }

    @Override
    @Transactional
    public void update(AlgTrain algTrain) {
        this.updateById(algTrain);
    }

    @Override
    public AlgTrain selectByMissName(String missName) {

        return algTrainMapper.selectByMissName(missName);

    }

    @Override
    public AlgTrain selectByPrimaryKey(Long algorithmTrainingId) {
        return algTrainMapper.selectByPrimaryKey(algorithmTrainingId);
    }


}
