package io.renren.modules.sys.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import io.renren.common.utils.PageUtils;
import io.renren.modules.sys.dao.algs.AlgorithmModelMapper;
import io.renren.modules.sys.dao.algs.AlgorithmModelVoMapper;
import io.renren.modules.sys.entity.algs.AlgorithmModel;
import io.renren.modules.sys.entity.algs.AlgorithmModel_Vo;
import io.renren.modules.sys.service.AlgsModelsService;
import io.renren.modules.sys.service.CategoryService;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Map;

/**
 * @Author: Ivan Q
 * @Date: 2021/6/8 9:38
 */
@Service("algsModelsService")
public class AlgsModelsServiceImpl extends ServiceImpl<AlgorithmModelMapper, AlgorithmModel> implements AlgsModelsService {
    @Autowired
    AlgorithmModelVoMapper algorithmModelVoMapper;
    @Autowired
    AlgorithmModelMapper algorithmModelMapper;
    @Autowired
    CategoryService categoryService;


    //获取模板列表,使用的是@select连表查询
    //但是怎么把搜索功能加进去呢？？？？
    @Override
    public PageUtils queryPageOnUse(Map<String, Object> params) {
        //获取搜索框搜索的模板名称
        String modelName = (String)params.get("modelName");
        String classificationtag = (String)params.get("classificationtag");
        byte form = 0;
        if(params.get("form") !=null){
            String t = (String) params.get("form");
            if(t.equals("1"))
                form = 1;
            else form = 2;
        }
        Long categoryId = categoryService.getByName(classificationtag);
        //获取当前登录的用户,如果是管理员的话，uid就为null
        Long uid = (Long) params.get("currentUserId");
        List<AlgorithmModel_Vo> modelInfo = new ArrayList<>();
        List<AlgorithmModel_Vo> modelAllInfo = new ArrayList<>();
        int pageSize = Integer.valueOf((String) params.get("limit"));
        int pageIndex = Integer.valueOf((String) params.get("page"));

//        此方法如果修改,则也需要修改下面的获取总页数的方法
        modelInfo = algorithmModelVoMapper.getModelInfo(new QueryWrapper<AlgorithmModel_Vo>()
                        .like(StringUtils.isNotBlank(modelName), "model_name", modelName)
                        //根据登录的用户展示他自己的模板
                        .eq(uid != null, "uid", uid)
                        .eq(classificationtag != null, "category_id", categoryId)
                        .eq(params.get("form") != null, "form", form)
                        .eq("is_approvaled",1)
//                使用limit用于分页,下面的函数没有limit进行分页
                        .last("limit " + (pageIndex - 1) * pageSize + "," + pageSize + ";")
        );

        //用于查全部，获取总页数。
        modelAllInfo = algorithmModelVoMapper.getModelInfo(new QueryWrapper<AlgorithmModel_Vo>()
                .like(StringUtils.isNotBlank(modelName), "model_name", modelName)
                //根据登录的用户展示他自己的模板
                .eq(uid != null, "uid", uid)
                .eq(classificationtag != null, "category_id", categoryId)
                .eq(params.get("form") != null, "form", form)
                .eq("is_approvaled",1)
        );

        for (AlgorithmModel_Vo vo : modelInfo) {
            vo.setCategoryName(categoryService.getById(vo.getCategoryId()).getCategoryName());
        }

        //弃用的方法
        //这个方法：根据当前登录的用户id返回只有当前用户的数据，但是没有结合搜索框搜索
        //templateInfo = algorithmTemplateVoMapper.getTemplateInfoById(uid);
        return new PageUtils(modelInfo, modelAllInfo.size(), pageSize, pageIndex);

    }

    /**
     * Description:获取所有算法信息，用于创建算法时的下拉选框
     * @param params
     * @return
     */
    @Override
    public List<AlgorithmModel_Vo> listAll(Map<String, Object> params){
        Long uid = (Long) params.get("currentUserId");
        List<AlgorithmModel_Vo> modelAllInfo = new ArrayList<>();
        modelAllInfo = algorithmModelVoMapper.getModelInfo(new QueryWrapper<AlgorithmModel_Vo>()
                //根据登录的用户展示他自己的模板
                .eq(uid != null, "uid", uid)
        );
        return modelAllInfo;
    }

    @Override
    public void deleteBatch(Long[] modelIds) {
        this.removeByIds(Arrays.asList(modelIds));
    }

    @Override
    public Long querycategoryId(Long tmpId) {
        return null;
    }

    @Override
    @Transactional
    public void update(AlgorithmModel model) {
            this.updateById(model);
    }

    @Override
    public AlgorithmModel selectByModelName(String modelName) {
        return algorithmModelMapper.selectByModelName(modelName);
    }


}
