package io.renren.modules.sys.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import io.minio.errors.*;
import io.renren.common.utils.MinIoUtils;
import io.renren.common.utils.PageUtils;
import io.renren.modules.sys.dao.algs.AlgorithmModelMapper;
import io.renren.modules.sys.dao.algs.AlgorithmModelVoMapper;
import io.renren.modules.sys.dao.dataset.DataSetMapper;
import io.renren.modules.sys.dao.dataset.DataSetVoMapper;
import io.renren.modules.sys.entity.algs.AlgorithmModel;
import io.renren.modules.sys.entity.algs.AlgorithmModel_Vo;
import io.renren.modules.sys.entity.algs.CategoryEntity;
import io.renren.modules.sys.entity.dataset.DataSet;
import io.renren.modules.sys.entity.dataset.DataSet_Vo;
import io.renren.modules.sys.service.CategoryService;
import io.renren.modules.sys.service.DataSetService;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.io.IOException;
import java.security.InvalidKeyException;
import java.security.NoSuchAlgorithmException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Map;

/**
 * @Author: Ivan Q
 * @Date: 2021/6/9 13:35
 */
@Service
public class DataSetServiceImpl extends ServiceImpl<DataSetMapper, DataSet>  implements DataSetService {

    @Autowired
    DataSetVoMapper dataSetVoMapper;
    @Autowired
    CategoryService categoryService;


    //获取模板列表,使用的是@select连表查询
    //但是怎么把搜索功能加进去呢？？？？
    @Override
    public PageUtils queryPageOnUse(Map<String, Object> params) {
        //获取搜索框搜索的模板名称
        String datasetName = (String)params.get("datasetName");
        String classificationtag = (String)params.get("classificationtag");
        byte status = 0;
        if(params.get("status") !=null){
            String t = (String) params.get("status");
            if(t.equals("1"))
                status = 1;
            else status = 2;
        }
        Long categoryId = categoryService.getByName(classificationtag);
        //获取当前登录的用户,如果是管理员的话，uid就为null
        Long uid = Long.parseLong((String) params.get("currentUserId")) ;
        List<DataSet_Vo> datasetInfo = new ArrayList<>();
        List<DataSet_Vo> datasetAllInfo = new ArrayList<>();
        int pageSize = Integer.valueOf((String) params.get("limit"));
        int pageIndex = Integer.valueOf((String) params.get("page"));

//        此方法如果修改,则也需要修改下面的获取总页数的方法
        datasetInfo = dataSetVoMapper.getModelInfo(new QueryWrapper<DataSet_Vo>()
                        .like(StringUtils.isNotBlank(datasetName), "dataset_name", datasetName)
                        //根据登录的用户展示他自己的模板
                        .eq(uid != null, "uid", uid)
                        .eq(classificationtag != null, "category_id", categoryId)
                        .eq(params.get("status") != null, "dataset_status", status)
                        .orderByDesc("dataset_id")
//                使用limit用于分页,下面的函数没有limit进行分页
                        .last("limit " + (pageIndex - 1) * pageSize + "," + pageSize + ";")

        );

        //用于查全部，获取总页数。
        datasetAllInfo = dataSetVoMapper.getModelInfo(new QueryWrapper<DataSet_Vo>()
                .like(StringUtils.isNotBlank(datasetName), "dataset_name", datasetName)
                //根据登录的用户展示他自己的模板
                .eq(uid != null, "uid", uid)
                .eq(classificationtag != null, "category_id", categoryId)
                .eq(params.get("status") != null, "dataset_status", status)
        );

        for (DataSet_Vo vo : datasetInfo) {
            final CategoryEntity categoryServiceById = categoryService.getById(vo.getCategoryId());
            if(categoryServiceById != null){
                vo.setCategoryName(categoryServiceById.getCategoryName());
            }else {
                vo.setCategoryName("待分类");
            }
        }

        //弃用的方法
        //这个方法：根据当前登录的用户id返回只有当前用户的数据，但是没有结合搜索框搜索
        //templateInfo = algorithmTemplateVoMapper.getTemplateInfoById(uid);
        return new PageUtils(datasetInfo, datasetAllInfo.size(), pageSize, pageIndex);

    }

    @Override
    public void deleteBatch(Long[] datasetIds) {
        this.removeByIds(Arrays.asList(datasetIds));
    }

    @Override
    public Long querycategoryId(Long datasetId) {
        return null;
    }

    @Override
    @Transactional
    public void update(DataSet dataSet) {
        this.updateById(dataSet);
    }

    @Override
    public String renameMino(String buckName, String newFileName, String srcBuckName, String srcFileName) {
        if (newFileName.equals(srcFileName) && buckName.equals(srcBuckName) ) return "200";
        try {
            MinIoUtils.copyFile(buckName,newFileName,srcBuckName,srcFileName);
            MinIoUtils.deleteFile(srcBuckName,srcFileName);
        } catch (IOException e) {
            e.printStackTrace();
            return srcFileName + " 文件服务器连接异常！！";
        } catch (InvalidKeyException e) {
            e.printStackTrace();
            return srcFileName + " 不存在！";
        } catch (InvalidResponseException e) {
            e.printStackTrace();
        } catch (InvalidBucketNameException e) {
            e.printStackTrace();
            return  srcBuckName + "不存在！！！";
        } catch (ErrorResponseException e) {
            e.printStackTrace();
            return   "执行失败";
        } catch (Exception e){
            return "内部错误";
        }

        return "200";

    }

}
